import type { PeerId } from '@libp2p/interface-peer-id';
import type { ConnectionManager } from '@libp2p/interface-connection-manager';
import type { TransportManager } from '@libp2p/interface-transport';
import type { Startable } from '@libp2p/interfaces/dist/src/startable.js';
import { EventEmitter } from '@libp2p/interfaces/events';
import type { PeerStore } from '@libp2p/interface-peer-store';
export interface RelayStoreComponents {
    peerId: PeerId;
    connectionManager: ConnectionManager;
    transportManager: TransportManager;
    peerStore: PeerStore;
}
export interface RelayStoreInit {
    /**
     * Multiple relays may be discovered simultaneously - to prevent listening
     * on too many relays, this value controls how many to attempt to reserve a
     * slot on at once. If set to more than one, we may end up listening on
     * more relays than the `maxReservations` value, but on networks with poor
     * connectivity the user may wish to attempt to reserve on multiple relays
     * simultaneously. (default: 1)
     */
    reservationConcurrency?: number;
    /**
     * How many discovered relays to allow in the reservation store
     */
    discoverRelays?: number;
}
export type RelayType = 'discovered' | 'configured';
export interface ReservationStoreEvents {
    'relay:not-enough-relays': CustomEvent;
    'relay:removed': CustomEvent<PeerId>;
}
export declare class ReservationStore extends EventEmitter<ReservationStoreEvents> implements Startable {
    #private;
    private readonly peerId;
    private readonly connectionManager;
    private readonly transportManager;
    private readonly peerStore;
    private readonly reserveQueue;
    private readonly reservations;
    private readonly maxDiscoveredRelays;
    private started;
    constructor(components: RelayStoreComponents, init?: RelayStoreInit);
    isStarted(): boolean;
    start(): Promise<void>;
    stop(): Promise<void>;
    /**
     * If the number of current relays is beneath the configured `maxReservations`
     * value, and the passed peer id is not our own, and we have a non-relayed connection
     * to the remote, and the remote peer speaks the hop protocol, try to reserve a slot
     * on the remote peer
     */
    addRelay(peerId: PeerId, type: RelayType): Promise<void>;
    hasReservation(peerId: PeerId): boolean;
    /**
     * Remove listen relay
     */
    removeRelay(peerId: PeerId): void;
}
//# sourceMappingURL=reservation-store.d.ts.map